import {StoreComponent} from "../../../../../app/storage/StoreComponent";
import {Gamesystem} from "../../game-model/gamesystems/Gamesystem";
import {ParsedParentGamesystems} from "./ParsedParentGamesystems";
import {ProductGamesystem} from "../../game-model/gamesystems/ProductGamesystem";
import {SimpleGamesystem} from "../../game-model/gamesystems/SimpleGamesystem";
import {StateParser} from "./StateParser";
import {ScriptAccount} from "../../game-model/scriptAccounts/ScriptAccount";
import {TransitionParser} from "./TransitionParser";

export class GamesystemParser {

  private parsedParentGamesystems: ParsedParentGamesystems[] = []
  private parsedGamesystems: Gamesystem<any, any>[] = []

  private scriptAccounts: ScriptAccount[]


  constructor(scriptAccounts: ScriptAccount[]) {
    this.scriptAccounts = scriptAccounts;
  }


  public parseStoredGamesystems(storedGamesystems: StoreComponent[]): Gamesystem<any, any>[] {
    storedGamesystems.forEach(storedGamesystem => {
      this.parseGamesystem(storedGamesystem)
    })

    return this.computeGamesystemStructure();
  }

  private parseGamesystem(storedGamesystem: StoreComponent) {
    const parsedGamesystemData = JSON.parse(storedGamesystem.jsonString)
    let parsedSystem: Gamesystem<any, any>
    if(parsedGamesystemData.childsystems != undefined) {
      parsedSystem = this.parseProductGamesystem(parsedGamesystemData)
    } else {
      parsedSystem = this.parseSimpleGamesystem(parsedGamesystemData)
    }
    this.parsedGamesystems.push(parsedSystem);
  }

  parseSimpleGamesystem(gamesystemData: any): SimpleGamesystem {
    const simpleGamesystem = new SimpleGamesystem(gamesystemData.componentName, gamesystemData.componentDescription)

    const stateParser = new StateParser(this.scriptAccounts);
    simpleGamesystem.states = stateParser.parseStates(gamesystemData.states)

    const transitionParser = new TransitionParser(simpleGamesystem.states, this.scriptAccounts)
    simpleGamesystem.transitions = transitionParser.parseTransitions(gamesystemData.transitions)
    return simpleGamesystem
  }

  parseProductGamesystem(gamesystemData: any) {
    const productGamesystem = new ProductGamesystem(gamesystemData.componentName, gamesystemData.componentDescription);
    const childsystemNames: string[] = []
    for(let i=0; i<gamesystemData.childsystems.length; i++) {
      childsystemNames.push(gamesystemData.childsystems[i].componentName)
    }
    this.parsedParentGamesystems.push(new ParsedParentGamesystems(productGamesystem, childsystemNames))

    return productGamesystem;
  }

  private computeGamesystemStructure(): Gamesystem<any, any>[] {
    const topGamesystems: Gamesystem<any, any>[] = []
    this.parsedGamesystems.forEach(parsedGamesystem => {
      const searchedParentsystem = this.findParentsystem(parsedGamesystem.componentName)
      if(searchedParentsystem != undefined) {
        searchedParentsystem.addChildGamesystem(parsedGamesystem)
        parsedGamesystem.parentGamesystem = searchedParentsystem
      } else {
        topGamesystems.push(parsedGamesystem)
      }
    })
    return topGamesystems
  }

  findParentsystem(childsystemName: string) {
    for(let i=0; i<this.parsedParentGamesystems.length; i++) {
      if(this.parsedParentGamesystems[i].childsystemNames.includes(childsystemName)) {
        return this.parsedParentGamesystems[i].parentGamesystem;
      }
    }
    return undefined
  }

}
